package org.steveshipway.blueprints.directive;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;
import org.steveshipway.blueprints.Blueprint;
import org.steveshipway.blueprints.Portfolio;

import net.minecraft.world.World;

// Minecraft 1.7
import org.steveshipway.blueprints.util17.*;
// Minecraft 1.8
//import org.steveshipway.blueprints.util18.*;
//import net.minecraft.util.BlockPos;

public class CallDirective extends Directive {
	private BlockPos location;
	private int rotation = 0;
	private Blueprint blueprint = null;
	private String bpname = "NULL";
	private Portfolio portfolio = null;
	
	public CallDirective(Blueprint blueprint, BlockPos offset, int r) {
		this.blueprint = blueprint;
		this.location = offset;
		this.rotation = r & 3;
	}
	public CallDirective(String definition,String myname, Portfolio portfolio,Logger logger) {
    	Pattern p = Pattern.compile("(?i)^(?:\\s*call)\\s*(\\w+)\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)\\s*(\\d*)");
    	Matcher m = p.matcher(definition);
    	this.portfolio = portfolio;
    	if( m.find() ) {
    		bpname = m.group(1);
    		try { 
    			this.location = new BlockPos(Integer.parseInt( m.group(2) ),Integer.parseInt( m.group(3) ),Integer.parseInt( m.group(4) ));
    		} catch( Exception e ) { 
    			this.error = true; 
    			this.location = new BlockPos(0,0,0);
    			logger.error("Invalid offset specified in call directive: "+definition);
    		}
    		if ( m.group(5) != "" ) {
    			try { this.rotation = Integer.parseInt( m.group(5) ) & 3; } catch( Exception e ) { }
    		}
    		if( ! m.group(1).equals(myname) && ! portfolio.defines(m.group(1)) ) {
    			this.error = true;
    			logger.error("Invalid blueprint name ["+m.group(1)+"] in definition!");
    		} else {
        		this.blueprint = portfolio.blueprint(m.group(1));
    		}
    	} else {
			this.error = true;
			logger.error("Invalid syntax in CALL definition!");    		
    	}
	}
	
	@Override
	public boolean build(World world, BlockPos location, int r, Logger logger) {
		BlockPos nc;
		int nr;
		boolean rv;
		if( this.error ) {
			logger.error("Error in call definition.  Cannot call ["+this.bpname+"].  Check definition.");
			return false;
		}
		if( this.blueprint == null && this.portfolio != null && this.bpname != null ) {
			this.blueprint = this.portfolio.blueprint(this.bpname);  // handle self-referential calls
		}
		if( this.blueprint == null ) {
			logger.error("Cannot call null blueprint.  This should never happen!");
			return false;
		}
		nc = rotateAndAdd( this.location, r, location );
		nr = ( this.rotation + r ) & 3;
		// CheshireCat.logger.info("Calling ["+blueprint+"] at location "+nc.getX()+","+nc.posY+","+nc.getZ());
		rv =  this.blueprint.build(world,nc,nr);
		if(!rv) {
			logger.error("CALL failed: ["+this.asText()+"]");
		}
		return rv;
	}
	
	@Override
	public String asText() {
		return (this.error?"# ":"")+"CALL "+this.bpname+" "+this.location.getX()+", "+this.location.getY()+", "+this.location.getZ()+" "+this.rotation;
	}
}
