package org.steveshipway.blueprints.directive;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;

import net.minecraft.block.Block;
import net.minecraft.block.BlockChest;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;



//Minecraft 1.7
import org.steveshipway.blueprints.util17.*;
//Minecraft 1.8
//import org.steveshipway.blueprints.util18.*;
//import net.minecraft.util.BlockPos;
//import net.minecraft.block.state.IBlockState;
import org.steveshipway.cheshirecat.CheshireCat;

class ChestItem {
	public String id = "minecraft:stick";
	public int mincount = 1;
	public int maxcount = 1;
	public int dv = 0;
	public int chance = 100;
	public NBTTagCompound tag = null;
	public Item item = null;
	public ChestItem(String s, NBTTagCompound itemtag, int d, int n1, int n2, int c) {
		id = s;
		this.tag = itemtag;
		mincount = n1;
		maxcount = n2;
		chance = c;
		if( chance < 0 ) { chance = 0; }
		if( chance > 100 ) { chance = 100; }
		if( mincount < 0) { mincount = 0; }
		if( maxcount < mincount ) { maxcount = mincount; }
		if( mincount > 64 ) { mincount = 64; }
		if( maxcount > 64 ) { maxcount = 64; }
		dv = d & 15;
		this.item = Util.getItemByNameOrId(s);
	}
}
public class ChestDirective extends Directive {
	private BlockPos offset;
	private int facing = 0; // facing direction
	private String contents; // for print/astext only
	private NBTTagCompound nbt; // pre-parsed NBT; if null, use list
	private LinkedList<ChestItem> items; // list of items with probabilities
	private boolean haserror = false;
	static Pattern itempattern = Pattern.compile("^\\s*(\\S+?)(?:\\((\\d+)\\))?(?:,(\\d+))?(?:,(\\d+))?(?:;(\\d+))?\\s*$");
	static Pattern nbtpattern = Pattern.compile("(?i)^(?:\\s*chest)?\\s*(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(\\d)\\s*(\\S+\\.nbt|\\{.*\\})\\s*$");
	static Pattern listpattern = Pattern.compile("(?i)^(?:\\s*chest)?\\s*(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(\\d)\\s*(.*)\\s*$");
	
	public ChestDirective(String definition, String bppath,Logger logger) {
		NBTTagCompound nbtItem = null;
		String item = null;
		int dv,minqty,maxqty,chance;
		Matcher m;
		// parse
		// contents can be filename.nbt, raw NBT, or list of ( itemname[,n[,m]][;prob], or itemfile.nbt[,n[,m]][;prob] )
		// turn this into either static nbt, or item list with each item a name+prob+qtyrange+optionaltag
		m = nbtpattern.matcher(definition);
		if ( m.find() ) {
			//logger.info("Chest definition matched explicit NBT: "+definition);
			try {
				this.nbt = Util.parseNBT(m.group(5),bppath,logger);
				this.offset = new BlockPos(Integer.parseInt(m.group(1)),Integer.parseInt(m.group(2)),Integer.parseInt(m.group(3)));
				this.facing = Integer.parseInt(m.group(4));
			} catch ( Exception e ) {
				logger.error("Invalid offset format in CHEST definition '"+definition+"'");
				this.haserror = true;
				return;
			}
			if( this.nbt.getString("id") == null ) {
				this.nbt.setString("id","Chest");
			} else if( this.nbt.getString("id").equals("") ) {
					this.nbt.setString("id","Chest");
			} else if( !this.nbt.getString("id").equals("Chest") ) {
				logger.error("Supplied NBT in '"+m.group(5)+"' is not for a Chest (id="+this.nbt.getString("id")+")");
				this.haserror = true;
			}
			return;
		}
		m = listpattern.matcher(definition);
		if(! m.find() ) {
			logger.error("Invalid format in CHEST definition '"+definition+"'");
			this.haserror = true;
			return;			
		}
		try {
			this.offset = new BlockPos(Integer.parseInt(m.group(1)),Integer.parseInt(m.group(2)),Integer.parseInt(m.group(3)));
			this.facing = Integer.parseInt(m.group(4));
		} catch ( Exception e ) {
			logger.error("Invalid offset format in CHEST definition '"+definition+"'");
			this.haserror = true;
			return;
		}
		this.contents = m.group(5);
		this.items = new LinkedList<ChestItem>();
		//logger.info("Chest definition matched content list: "+this.contents);
		// is a list of items
		for(String itemdef : this.contents.split("\\s+") ) {
			NBTTagCompound tag = null;
			//CharSequence itemdefcs = itemdef;
			if( itemdef.equals("") ) { continue; }

			m = itempattern.matcher( itemdef );
			if( m.find() ) {
				item = m.group(1);
				//logger.info("Chest item matched pattern: "+item);
				if( m.group(2)!= null ) {
					dv = Integer.parseInt( m.group(2) );
				} else { dv = 0; }
				if( m.group(3)!= null ) {
					minqty = Integer.parseInt(m.group(3));
				} else { minqty=1; }
				if( m.group(4)!= null ) {
					maxqty = Integer.parseInt(m.group(4));
				} else { maxqty=minqty; }
				if( m.group(5)!= null ) {
					chance = Integer.parseInt(m.group(5));
				} else { chance=100; }
				if((dv < 0) || (dv>15)) { dv = 0; }
				if(minqty < 0) { minqty = 1; }
				if(chance<0 || chance>100) { chance = 100; }
				if( item.endsWith(".nbt") || item.startsWith("{") ) {
					nbtItem = Util.parseNBT(item,bppath,logger);
					if( nbtItem!=null ) {
						//extract id, dv and tag from the NBT
						item = nbtItem.getString("id");
						dv = nbtItem.getShort("Damage");
						if( nbtItem.hasKey("tag") ) {
							tag = (NBTTagCompound) nbtItem.getTag("tag").copy();
						}
					} else {
						logger.error("Invalid NBT in item definition '"+item+"'");
						this.haserror = true;
						continue;
					}
				} else {
					if( !item.contains(":") ) { item = "minecraft:"+item; }

					if( Util.getItemByNameOrId(item) == null ) { // check for validity
						logger.error("Invalid item name defined "+item);
						this.haserror = true;
						continue;
					}
				}

				this.items.add(new ChestItem(item,tag,dv,minqty,maxqty,chance));
				
			} else {
				logger.error("Invalid chest item definition: '"+itemdef+"'");
				this.haserror = true;
				return;
			}
		}
	}
	
	@Override
	public boolean build(World world, BlockPos location, int r, Logger logger) {
		IBlockState blockState = null;
		BlockPos blockPos = new BlockPos(location);
		int dv = 2; 
		boolean rv;
		NBTTagCompound nbtTag = null;
		
		if( world.isRemote ) {
    		logger.error("Block placement called with remote world?  Should never happen.");
			return false;
		}

		// rotate to correct DV for facing
		switch( (r+facing)&3 ) {
			case 1:
				dv = 5;
				break;
			case 2:
				dv = 3;
				break;
			case 3:
				dv = 4;
				break;
			default:
				dv = 2;
				break;
		}
    	// temporary
		//CheshireCat.portfolio.getlogger().info("Calculating dv="+dv);
		
		// get block position, rotated
		blockPos = rotateAndAdd(this.offset,r,location);

		// set the block to a chest...
		blockState = Util.getState(Blocks.chest,dv);
		rv = Util.setBlockState(world,blockPos,blockState);
		if( !rv ) {
			logger.warn("Failed to place chest in the world at "+blockPos.getX()+","+blockPos.getY()+","+blockPos.getZ());		
			return false;
		}
		if(this.haserror) {
			logger.warn("Chest contents definition has errors");
			// return false;
		}
		// now we have the block to set, but need to add the tile entity for the contents
		if( this.nbt != null ) {
			logger.info("Setting chest with explicit contents");
			// static NBT pre-parsed
			nbtTag = (NBTTagCompound) this.nbt.copy();
		} else {
			logger.info("Setting chest with random contents");
			// random list
			nbtTag = chestTag(logger);
		}
		try {
			logger.info("Creating tile entity");
//			Util.setTileEntity(world, blockPos, TileEntity.createAndLoadEntity(nbtTag));
			Util.setTileEntity(world, blockPos, Util.fixChestNbt(logger,nbtTag));
		} catch(Exception e) {
			logger.warn("Unable to create TileEntity for chest, may be illegal NBT: "+e.getMessage());		
			return false;
		}
		
		return true;
	}
	
	@Override
	public String asText() {
		String s = "CHEST ";
		if( this.haserror ) {
			s = "# CHEST ";
		}
		s += " " + offset.getX() + "," + offset.getY() + "," + offset.getZ() + " " + facing;
		if( contents != null ) {
			s += " " + contents;
		}
		return s;
	}
	
	private NBTTagCompound chestTag(Logger logger) {
		// render the this.list as a random NBTTagCompound
		NBTTagCompound nbtTag  = new NBTTagCompound();
		NBTTagCompound nbtItemTag;
		NBTTagCompound nbtItemOpt = null;
		NBTTagList nbtItems = new NBTTagList();
		int slot = 0;
		int stacksize = 1;
		Item i = null;
		ItemStack itemstack = null;
		Random rand = new Random();
		ChestItem ci;

		// predefined NBT
		if( this.nbt != null ) { return this.nbt; }
		
		// everything has these
		nbtTag.setString("id","Chest");
		nbtTag.setString("Lock","");
		
		if( this.items == null ) { //empty chest
			return nbtTag;
		}
		if( this.nbt != null ) {
			return this.nbt;       // preparsed
		}
		// now the items
		Iterator ii = this.items.iterator();
		while( ii.hasNext() ) {
			ci = (ChestItem) ii.next();
			i = Util.getItemByNameOrId(ci.id);
			nbtItemOpt = null;
			if( i == null ) {
				// invalid item!
				logger.warn("Invalid item name in chest contents: "+ci.id);
				continue;
			}
			nbtItemOpt = ci.tag;
			itemstack = new ItemStack(i,ci.dv);
			if( ci.maxcount <= ci.mincount ) {
				stacksize = ci.maxcount;
			} else {
				stacksize = ci.mincount + rand.nextInt(ci.maxcount - ci.mincount);
			}
			if( stacksize > i.getItemStackLimit(itemstack) ) { stacksize = i.getItemStackLimit(itemstack); }
			if( stacksize < 1 ) { continue; } // skip this item as we have 0
			// now add this item to the NBT
			nbtItemTag = new NBTTagCompound();
			nbtItemTag.setByte("Count", (byte) stacksize);
			nbtItemTag.setByte("Slot", (byte) slot);
			nbtItemTag.setShort("Damage", (short) ci.dv);
			nbtItemTag.setString("id", ci.id);
			if( nbtItemOpt != null ) {
				nbtItemTag.setTag("tag",nbtItemOpt);
			}

			// add this itemstack NBT to the Chest's items list
			nbtItems.appendTag(nbtItemTag);			
			slot += 1;
		}
		nbtTag.setTag("Items",nbtItems);			
		
		return nbtTag;
	}
}
