package org.steveshipway.blueprints.util18;

// All Minecraft 1.8 utility functions should be in here!

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;

import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.nbt.CompressedStreamTools;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.NBTException;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;

// Minecraft 1.7
//import org.steveshipway.blueprints.compat.*;
//import cpw.mods.fml.common.registry.GameRegistry;
// Minecraft 1.8
import net.minecraft.block.BlockLever.EnumOrientation;
import net.minecraft.block.BlockRailBase.EnumRailDirection;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyDirection;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.properties.PropertyInteger;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.EnumFacing.Axis;


public class Util {
	// Minecraft 1.8
    public static final PropertyEnum ORIENTATION = PropertyEnum.create("facing", EnumOrientation.class);
    public static final PropertyDirection FACING = PropertyDirection.create("facing");
    public static final PropertyDirection AXIS = PropertyDirection.create("axis");
    public static final PropertyInteger ROTATION = PropertyInteger.create("rotation",0,15);
    public static final PropertyBool NORTH = PropertyBool.create("north");
    public static final PropertyBool SOUTH = PropertyBool.create("south");
    public static final PropertyBool EAST  = PropertyBool.create("east");
    public static final PropertyBool WEST  = PropertyBool.create("west");
    public static final PropertyEnum SHAPE = PropertyEnum.create("shape",EnumRailDirection.class);
    
	public static Pattern nbtfile = Pattern.compile("\\s*(\\S+\\.nbt)\\s*");
	
	public Util() {
	}
	
    public static IBlockState rotateState( IBlockState blockstate, int r ) {
    	EnumFacing facing;

    	EnumRailDirection raildirection;
    	Axis axis;
    	Integer rotation;
    	IBlockState newstate = blockstate;		
    	Map props = blockstate.getProperties();
    	// rotation for most blocks
    	if( props.containsKey(FACING) ) {      
    		// seems to fail for levers?
    		facing = (EnumFacing) blockstate.getValue(FACING);
            //logger.info("Rotating a " + blockstate.getBlock().toString() + " facing."+facing.toString()+" + "+r);
    		if( facing == EnumFacing.UP || facing == EnumFacing.DOWN ) {
    			// up and down cannot be rotated
    			return blockstate;
    		}
    		if( r == 1 ) {
        		facing = facing.rotateY();
    		} else if( r == 2 ) {
    			facing = facing.rotateY();
    			facing = facing.rotateY();
    		} else if( r == 3 ) {
    			facing = facing.rotateYCCW();
    		}
    		newstate = blockstate.withProperty(FACING, facing);    	
    		return newstate;
    	}
    	if( props.containsKey(ORIENTATION) ) {      
    			EnumOrientation leverfacing;
    			int ir = r;
    			HashMap<EnumOrientation,EnumOrientation> levermap = new HashMap<EnumOrientation,EnumOrientation>();
        		levermap.put(EnumOrientation.NORTH,EnumOrientation.EAST);
        		levermap.put(EnumOrientation.EAST,EnumOrientation.SOUTH);
        		levermap.put(EnumOrientation.SOUTH,EnumOrientation.WEST);
        		levermap.put(EnumOrientation.WEST,EnumOrientation.NORTH);
        		levermap.put(EnumOrientation.UP_X,EnumOrientation.UP_Z);
        		levermap.put(EnumOrientation.DOWN_X,EnumOrientation.DOWN_Z);
        		levermap.put(EnumOrientation.UP_Z,EnumOrientation.UP_X);
        		levermap.put(EnumOrientation.DOWN_Z,EnumOrientation.DOWN_X);
        		
        		leverfacing = (EnumOrientation) blockstate.getValue(ORIENTATION);
                //logger.info("Rotating a lever facing."+leverfacing.toString()+" by "+r);
        		while( ir > 0 ) {
        			ir -= 1;
        			leverfacing = levermap.get(leverfacing);
        		}
                //logger.info("Now have facing."+leverfacing.toString());
	    		newstate = blockstate.withProperty(ORIENTATION, leverfacing);
	    		return newstate;
    	}
    	// rotation for logs, hay bales
    	if( props.containsKey(AXIS) ) {      
    		axis = (Axis) blockstate.getValue(AXIS);
//            logger.info("Rotating an axis."+axis.toString());
    		if( r == 1 || r == 3) {
    			if( axis == Axis.X ) {
    				axis = Axis.Z;
    			} else if( axis == Axis.Z ) {
    				axis = Axis.X;
    			}
    		}
    		newstate = blockstate.withProperty(AXIS, axis);
    		return newstate;
    	}
    	// rotation for banners and signs
    	if( props.containsKey(ROTATION) ) {      
    		newstate = blockstate;
    		rotation = (Integer) blockstate.getValue(ROTATION);
    		rotation = rotation + 4; // 4 positions in a quarter turn
    		if( rotation > 15 ) { rotation -= 16; }
    		newstate = blockstate.withProperty(ROTATION, rotation);
    		return newstate;
    	}
    	// rotation for vines
    	if( props.containsKey(NORTH) ) {      
    		int ir = r;
    		Boolean north = (Boolean) blockstate.getValue(NORTH);
    		Boolean south = (Boolean) blockstate.getValue(SOUTH);
    		Boolean east  = (Boolean) blockstate.getValue(EAST);
    		Boolean west  = (Boolean) blockstate.getValue(WEST);
    		Boolean t;
    		while( ir > 0 ) {
    			ir -= 1;
    			t = east; east = north; north = west; west = south; south = t;
    		}
    		newstate = blockstate.withProperty(NORTH, north).withProperty(SOUTH, south).withProperty(EAST, east).withProperty(WEST, west);
    		return newstate;
    	}
    	// rotation for train tracks; very weird
    	if( props.containsKey(SHAPE) ) {      
    		int ir = r;
    		HashMap<EnumRailDirection,EnumRailDirection> railmap = new HashMap<EnumRailDirection,EnumRailDirection>();
    		railmap.put(EnumRailDirection.NORTH_SOUTH,EnumRailDirection.EAST_WEST);
    		railmap.put(EnumRailDirection.EAST_WEST,EnumRailDirection.NORTH_SOUTH);
    		railmap.put(EnumRailDirection.ASCENDING_NORTH,EnumRailDirection.ASCENDING_EAST);
    		railmap.put(EnumRailDirection.ASCENDING_EAST,EnumRailDirection.ASCENDING_SOUTH);
    		railmap.put(EnumRailDirection.ASCENDING_SOUTH,EnumRailDirection.ASCENDING_WEST);
    		railmap.put(EnumRailDirection.ASCENDING_WEST,EnumRailDirection.ASCENDING_NORTH);
    		railmap.put(EnumRailDirection.NORTH_EAST,EnumRailDirection.SOUTH_EAST);
    		railmap.put(EnumRailDirection.SOUTH_EAST,EnumRailDirection.SOUTH_WEST);
    		railmap.put(EnumRailDirection.SOUTH_WEST,EnumRailDirection.NORTH_WEST);
    		railmap.put(EnumRailDirection.NORTH_WEST,EnumRailDirection.NORTH_EAST);

    		raildirection = (EnumRailDirection) blockstate.getValue(SHAPE);
    		while( ir > 0 ) {
    			ir -= 1;
    			raildirection = railmap.get(raildirection);
    		}
    		newstate = blockstate.withProperty(SHAPE, raildirection);
    		return newstate;
    	}
    	// rotation for mushroom blocks; really weird
    	// XXXX to be done if I can work out how
		return blockstate;
    }
    
    // Utility function.  Convert "{nbt}" or "file.nbt" into a parsed NBT tag
    public static NBTTagCompound parseNBT(String s, String bppath, Logger logger) {
    	NBTTagCompound nbtTag = null;
    	Matcher m = nbtfile.matcher(s);
    	File nbtfile;
    	if( m.find() ) {
    		// we have an NBT file
    		if( s.startsWith(File.separator) || bppath == null ) {
    			nbtfile = new File(m.group(1));
    		} else {
    			nbtfile = new File( bppath+File.separator+"nbt"+File.separator+m.group(1) );
    		}
    		if( ! nbtfile.exists() ) {
				logger.error("Unable to read file "+m.group(1)+" : File does not exist in "+bppath+File.separator+"nbt"+File.separator);
    			return null;
    		} 
    		try {
				nbtTag = CompressedStreamTools.readCompressed(new FileInputStream(nbtfile));
			} catch (IOException e) {
				logger.error("Unable to read file "+m.group(1)+" : "+e.toString());
				return null;
			}
    	} else {
			try {
				// seek out the only function in JsonToNBT that takes a string and returns an NBTTag
				// Minecraft 1.7
		    	//nbtTag = (NBTTagCompound) JsonToNBT.func_150315_a(s); // forge 1.7.10
		    	// Minecraft 1.8
				nbtTag = (NBTTagCompound) JsonToNBT.func_180713_a(s); // forge 1.8
			} catch (NBTException e) {
				nbtTag = null;
				if( logger != null ) {
					logger.error(s);
					logger.error("Unable to parse NBT: "+e.getMessage());
				}
				return null;
			} 
    	}
    	return nbtTag;
    }
    
    // Abstract out getBlock() for portability
    public static Block getBlock(World world, BlockPos location) {
    	Block blk;
    	// Minecraft 1.8
    	blk =  world.getBlockState(location).getBlock();
    	// Minecraft 1.7
    	//blk =  world.getBlock(location.getX(),location.getY(),location.getZ());
    	return blk;
    }
    
    public static IBlockState getState(Block b,int dv) {
    	IBlockState bs;
    	
    	// Minecraft 1.7
    	//bs = new IBlockState(b, dv);
    	// Minecraft 1.8
    	bs = b.getBlockState().getBaseState().getBlock().getStateFromMeta(dv);
    	
    	return bs;
    }
    public static IBlockState getBlockState(World world, BlockPos blockPos) {
    	IBlockState bs;
    	// Minecraft 1.7
		//Block curBlock = world.getBlock(blockPos.getX(), blockPos.getY(), blockPos.getZ());
		//int curDv = world.getBlockMetadata(blockPos.getX(), blockPos.getY(), blockPos.getZ());
		//bs = new IBlockState(curBlock,curDv);
    	// Minecraft 1.8
		bs = world.getBlockState(blockPos);
		 
		return bs;
    }
    public static boolean setBlockState(World world, BlockPos blockPos, IBlockState bs) {
    	boolean rv;
    	
    	// Minecraft 1.7
    	//rv = world.setBlock(blockPos.getX(),blockPos.getY(),blockPos.getZ(),bs.getBlock(),bs.getDv(),2);
    	// Minecraft 1.8
    	rv = world.setBlockState(blockPos,bs,2);
    	
    	return rv;
    }
    public static int getDvFromState(IBlockState bs) {
    	int dv;
    	
    	// Minecraft 1.7
    	//dv = bs.getDv();
    	// Minecraft 1.8
    	dv = bs.getBlock().getMetaFromState(bs);
    	
    	return dv;
    }
    public static void setTileEntity(World world,BlockPos blockPos,TileEntity te) {
		// Minecraft 1.8
		world.removeTileEntity(blockPos); // should we instead get and merge?
		world.setTileEntity(blockPos,te);
		// Minecraft 1.7
		//world.removeTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ()); // should we instead get and merge?
		//world.setTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ(),te);

    }
    public static TileEntity getTileEntity(World world, BlockPos blockPos) {
    	// Minecraft 1.7
    	//return world.getTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ());
    	// Minecraft 1.8
    	return world.getTileEntity(blockPos);
    }
    public static Item getItemByNameOrId(int id) {
    	// Minecraft 1.7
    	//return Item.getItemById(id);
    	// Minecraft 1.8
    	return Item.getByNameOrId(""+id);    	
    }
    public static Item getItemByNameOrId(String s) {
    	// Minecraft 1.7
        //String modId = "minecraft";
        //String name;
        //if (s.indexOf(":") != -1) {
        //    modId = s.substring(0, s.indexOf(":"));
        //}
        //name = s.substring(s.indexOf(":") + 1);
        //return GameRegistry.findItem(modId, name); // Will return null if no item is found.
    	// Minecraft 1.8
    	return Item.getByNameOrId(s);
    }
}
